#include "mpffn.h"
#include "mpffn_priv.h"
#include "xmpn.h"

/*
@function: mpffn_add
@section: mpffn/arith
@proto: void mpffn_add(mp_ptr r, mp_srcptr x, mp_srcptr y, const mpffn_field *ff)
@desc: Set $r$ to $x+y$ in \ff. 
*/

void mpffn_add(mp_ptr r, mp_srcptr x, mp_srcptr y, const mpffn_field *ff)
{
  mpn_add_n(r,x,y,ff->size_elt);
  mpffn_canon_1(r,r,ff);
}

/*
@function: mpffn_pol_add
@section: mpffn/pol
@proto: void mpffn_pol_add(mp_ptr r, mp_srcptr x, mp_srcptr y, mp_size_t lt, const mpffn_field *ff)
@desc: Set $\{r,lt\}$ to $\{x,lt\}+\{y,lt\}$ in \fX. 
*/

void mpffn_pol_add(mp_ptr r, mp_srcptr x, mp_srcptr y, mp_size_t lt, const mpffn_field *ff)
{
  mpn_add(r,x,ff->size_elt,y,lt*ff->lp);
  mpffn_canon_1(r,r,ff);
}

/*
@function: mpffn_sub
@section: mpffn/arith
@proto: void mpffn_sub(mp_ptr r, mp_srcptr x, mp_srcptr y, const mpffn_field *ff)
@temp: sub=ff->size_elt
@desc: Set $r$ to $x-y$ in \ff. 
*/

void mpffn_sub(mp_ptr r, mp_srcptr x, mp_srcptr y, const mpffn_field *ff)
{
  mp_ptr z=ff->tmp->sub;
  mpffn_neg(z,y,ff);
  mpffn_add(r,x,z,ff);
}

/*
@function: mpffn_pol_sub
@section: mpffn/pol
@proto: void mpffn_pol_sub(mp_ptr r, mp_srcptr x, mp_srcptr y, mp_size_t lt, const mpffn_field *ff)
@temp: sub=ff->size_elt
@desc: Set $\{r,lt\}$ to $\{x,lt\}-\{y,lt\}$ in \fX. 
*/

void mpffn_pol_sub(mp_ptr r, mp_srcptr x, mp_srcptr y, mp_size_t lt, const mpffn_field *ff)
{
  mp_ptr z=ff->tmp->sub;
  mpffn_pol_neg(z,y,lt,ff);
  mpffn_pol_add(r,x,z,lt,ff);
}
