#include "mpffn.h"
#include "mpffn_priv.h"
#include "xmpn.h"

/*
@function: mpffn_add_ui
@section: mpffn/arith
@proto: void mpffn_add_ui(mp_ptr r, mp_srcptr x, unsigned long y, const mpffn_field *ff)
@desc: Set $r$ to $x+y$ in \ff. $0\leq y <p$  must hold.
*/

void mpffn_add_ui(mp_ptr r, mp_srcptr x, unsigned long y, const mpffn_field *ff)
{
  mpn_add_1(r,x,ff->size_elt,y);
  if (mpn_cmp(r,ff->p,ff->lp)>=0)
    mpn_sub_n(r,r,ff->p,ff->lp);
}
/*
@function: mpffn_sub_ui
@section: mpffn/arith
@proto: void mpffn_sub_ui(mp_ptr r, mp_srcptr x, unsigned long y, const mpffn_field *ff)
@desc: Set $r$ to $x-y$ in \ff. $0\leq y <p$  must hold.
*/

void mpffn_sub_ui(mp_ptr r, mp_srcptr x, unsigned long y, const mpffn_field *ff)
{
  mpn_sub_1(r,ff->p,ff->lp,y);
  mpffn_pol_add(r,x,r,1,ff);
}

/*
@function: mpffn_pol_neg
@section: mpffn/pol
@proto: void mpffn_pol_neg(mp_ptr r, mp_srcptr x, mp_size_t lt, const mpffn_field *ff)
@desc: Set $\{r,lt\}$ to $-\{x,lt\}$ in \fX.
*/

void mpffn_pol_neg(mp_ptr r, mp_srcptr x, mp_size_t lt, const mpffn_field *ff)
{
  long i;
  for(i=0; i<lt; i++, x+=ff->lp, r+=ff->lp)
  { 
    if (!xmpn_is_zero(x,ff->lp))
      mpn_sub_n(r,ff->p,x,ff->lp);
    else if (x!=r)
      xmpn_zero(r,ff->lp);
  }
}
