#include "mpffn.h"
#include "xmpn.h"

/*
@function: mpffn_cmp
@section: mpffn/cmp
@proto: int mpffn_cmp(mp_srcptr x, mp_srcptr y, const mpffn_field *ff)
@desc: Compare $x$ and $y$ in \ff.  Return $0$ if $x=y$, or $1$ if $x \neq y$.
*/


int mpffn_cmp(mp_srcptr x, mp_srcptr y, const mpffn_field *ff)
{
  /*CANONICALIZE*/
  return !!mpn_cmp(x,y,ff->size_elt);
}

/*
@function: mpffn_sgn
@section: mpffn/cmp
@proto: int mpffn_sgn(mp_srcptr x, const mpffn_field *ff)
@desc: Return $1$ if $x\neq 0$, $0$ if $x=0$.
*/

int mpffn_sgn(mp_srcptr x, const mpffn_field *ff)
{
  /*CANONICALIZE*/
  return !xmpn_is_zero(x, ff->size_elt);
}

/*
@function: mpffn_pol_sgn
@section: mpffn/pol
@proto: int mpffn_pol_sgn(mp_srcptr x, mp_size_t l, const mpffn_field *ff)
@desc: Return $1$ if $\{x, l\}\neq 0$, $0$ if $x=0$.
*/

int mpffn_pol_sgn(mp_srcptr x, mp_size_t l, const mpffn_field *ff)
{
  /*CANONICALIZE*/
  return !xmpn_is_zero(x, l*ff->lp);
}

