#include "mpffn.h"
#include "xmpn.h"

/*
@function: mpffn_copy
@section: mpffn/assign
@proto: void mpffn_copy(mp_ptr r, mp_srcptr x, const mpffn_field *ff)
@desc: Set $r$ to $x$ in \ff.
*/
void mpffn_copy(mp_ptr r, mp_srcptr x, const mpffn_field *ff)
{
  if (r!=x) xmpn_copy(r,x,ff->size_elt);
}

/*
@function: mpffn_pol_copy
@section: mpffn/pol
@proto: void mpffn_pol_copy(mp_ptr r, mp_srcptr x, mp_size_t l, const mpffn_field *ff)
@desc: Set $\{r,l\}$ to $\{x,l\}$ in \fX.
*/
void mpffn_pol_copy(mp_ptr r, mp_srcptr x, mp_size_t l, const mpffn_field *ff)
{
  if (r!=x) xmpn_copy(r,x,l*ff->lp);
}

/*
@function: mpffn_zero
@section: mpffn/assign
@proto: void mpffn_zero(mp_ptr r, const mpffn_field *ff)
@desc: Set $r$ to $0$ in \ff.
*/
void mpffn_zero(mp_ptr r, const mpffn_field *ff)
{
  xmpn_zero(r,ff->size_elt);
}

/*
@function: mpffn_pol_zero
@section: mpffn/pol
@proto: void mpffn_pol_zero(mp_ptr r, mp_size_t l, const mpffn_field *ff)
@desc: Set $r$ to $0$ in \ff.
*/
void mpffn_pol_zero(mp_ptr r, mp_size_t l, const mpffn_field *ff)
{
  xmpn_zero(r,l*ff->lp);
}

/*
@function: mpffn_one
@section: mpffn/assign
@proto: void mpffn_one(mp_ptr r, const mpffn_field *ff)
@desc: Set $r$ to $1$ in \ff.
*/
void mpffn_one(mp_ptr r, const mpffn_field *ff)
{
  xmpn_zero(r,ff->size_elt);
  r[0]=1;
}
