#include "mpffn.h"
#include "mpffn_priv.h"
#include "xmpn.h"

/*
@function: mpffn_div
@section: mpffn/arith
@proto: void mpffn_div(mp_ptr r, mp_srcptr x, mp_srcptr y, const mpffn_field *ff)
@temp: div=ff->size_elt
@desc: Set $r$ to $x/y$ in \ff.
*/
void mpffn_div(mp_ptr r, mp_srcptr x, mp_srcptr y, const mpffn_field *ff)
{
  mp_ptr z=ff->tmp->div;
  mpffn_inv(z,y,ff);
  mpffn_mul(r,x,z,ff);
}

/*
@function: mpffn_inv
@section: mpffn/arith
@proto: void mpffn_inv(mp_ptr r, mp_srcptr x, const mpffn_field *ff)
@temp: gcdext_u=ff->size_elt
       gcdext_v=ff->size_elt
       gcdext_v1=ff->size_elt
       gcdext_d=ff->size_elt
       gcdext_d1=ff->size_elt
       gcdext_q=ff->size_elt
       gcdext_r=ff->size_elt
@desc: Set $r$ to $x^{-1}$ in \ff.
*/
void mpffn_inv(mp_ptr g, mp_srcptr x, const mpffn_field *ff)
{
  mp_ptr d=ff->tmp->gcdext_d;
  mp_ptr d1=ff->tmp->gcdext_d1;
  mp_ptr u=ff->tmp->gcdext_u;
  mp_ptr v=ff->tmp->gcdext_v;
  mp_ptr v1=ff->tmp->gcdext_v1;
  mp_ptr q=ff->tmp->gcdext_q;
  mp_ptr r=ff->tmp->gcdext_r;
  mp_size_t ld,ld1;
  mp_size_t lv,lv1,lu;
  mpffn_copy(d1,x,ff); ld1=mpffn_length(d1,ff); ld=ff->lt+1;
  mpffn_zero(v,ff);
  mpffn_one(v1,ff); lv1=1;
  mpffn_pol_diveuc(q,r,ff->pol,ld,d1,ld1,ff);
  lv=ld-ld1+1;
  mpffn_pol_neg(v,q,lv,ff);
  u=v; v=v1; v1=u; 
  lu=lv; lv=lv1; lv1=lu;
  u=r; r=d; d=d1; d1=u; 
  ld=ld1; ld1=mpffn_pol_length(d1,ld1-1,ff);
  while (mpffn_pol_sgn(d1,ld1,ff))
  {
    mp_size_t lq=ld-ld1+1,lr;
    if (lq>0)
    {
      mpffn_pol_diveuc(q,r, d,ld, d1,ld1, ff);
      lr=mpffn_pol_length(r,ld1-1,ff);
      mpffn_pol_mul(q, q, lq, v1, lv1, ff); 
      lv=lq+lv1-1;
      mpffn_pol_sub(v,v,q,lv,ff);
    } 
    else
    {
      mpffn_pol_copy(r,d,ld,ff);
      lr=ld;
    }
    u=v; v=v1; v1=u; 
    lu=lv; lv=lv1; lv1=lu;
    u=r; r=d; d=d1; d1=u;
    lu=lr; lr=ld; ld=ld1; ld1=lu;
  }
  mpffn_invmod(d,d,ff);
  mpffn_mul_mpn(g,v,d,ff);
}
