#include "mpffn.h"
#include "mpffn_priv.h"
#include "xmpn.h"

/*
@function: mpffn_pol_diveuc
@section: mpffn/pol
@proto: void mpffn_pol_diveuc(mp_ptr q, mp_ptr r, mp_srcptr x, mp_size_t lx, mp_srcptr y, mp_size_t ly, const mpffn_field *ff)
@temp: diveuc_inv=ff->size_p
       diveuc_x=ff->size_elt+ff->lp
       diveuc_y=ff->size_elt
       diveuc=ff->size_elt
@desc: Set $q$ and $r$ so that $\{x,lx\}=\{y,ly\}\{q,lx-ly+1\}+\{r,ly-1\}$.
This function assumes  $lx-ly+1>0$.
*/

void mpffn_pol_diveuc(mp_ptr q, mp_ptr r, mp_srcptr x, mp_size_t lx, mp_srcptr y, mp_size_t ly, const mpffn_field *ff)
{
  mp_ptr x1=ff->tmp->diveuc_x;
  mp_ptr y1=ff->tmp->diveuc_y;
  mp_ptr z=ff->tmp->diveuc;
  mp_limb_t inv;
  mp_size_t lq=lx-ly+1;
  long i;
  mpffn_invmod(&inv,y+(ly-1),ff);       /* inv = 1/lc(y) */
  mpffn_pol_mul_ui(y1,y,ly,inv,ff);    /* y1 = inv*y */
  mpffn_pol_copy(x1,x,lx,ff);           /* x1 = x */ 
  for(i=lq-1; i>=0; i--)
  {
    q[i]=x1[i+(ly-1)];                  /* q[i] = lc(x1)*/
    if (q[i]==0) continue;
    mpffn_pol_mul_ui(z,y1,ly,q[i],ff);  /* z  = y1*q[i] */
    mpffn_pol_sub(x1+i,x1+i,z,ly,ff);   /* x1 = x1 - z*X^i */
  }
  mpffn_pol_mul_ui(q,q,lq,inv,ff);     /* q = q * inv */
  mpffn_pol_copy(r,x1,ly-1,ff);         /* r = x1 */
}
