#include "mpffn.h"
#include "mpffn_priv.h"
#include "xmpn.h"

/* mpn_gcdext destroys its input and needs an extra limb*/

/* 
@function: mpffn_invmod
@section: mpffn/misc
@proto: void mpffn_invmod(mp_ptr r, mp_srcptr x, const mpffn_field *ff)
@temp: invp_ca=ff->size_p+1
  invp_cb=ff->size_p+1
  invp_d=ff->size_p+1
  invp_r=ff->size_p+1
@desc: Set r to $x^-1$ where $x$ is in $\mathbb{F}_p$. x must be $0<x<p$.
*/

void mpffn_invmod(mp_ptr r, mp_srcptr x, const mpffn_field *ff)
{
  mp_ptr ca=ff->tmp->invp_ca;
  mp_ptr cb=ff->tmp->invp_cb;
  mp_ptr d=ff->tmp->invp_d;
  mp_ptr r2=ff->tmp->invp_r;
  mp_size_t lr,l;
  mp_size_t sp=ff->size_p;
  /* General case: use gcdext(a+b, b) since mpn_gcdext require S1>=S2 */
  if (!mpffn_sgn(x,ff)) {volatile int zero=0; l=1/zero;}
  mpn_add(ca, ff->p, sp, x, sp);
  xmpn_copy(cb, ff->p, sp);
  l=mpn_gcdext(d, r2, &lr, ca, sp, cb, sp);
  if (lr<0)
  {
    lr=-lr;
    mpn_sub(r, ff->p, sp, r2, lr);
  }
  else
    xmpn_copy(r, r2, lr);
  xmpn_zero(r+lr, ff->lp-lr);
}
