#include "mpffn.h"
#include "mpffn_priv.h"
#include "xmpn.h"

/*
@function: mpffn_pol_montgomery
@section: mpffn/private
@proto: void mpffn_pol_montgomery(mp_ptr r, mp_srcptr x, mp_size_t l, const mpffn_field *ff)
@temp: mg_q=ff->size_sqr
       mg_xx=ff->size_elt
@desc: Compute $x/polrecip(P)+O(x^n)$
*/

void mpffn_pol_montgomery(mp_ptr r, mp_srcptr x, mp_size_t l, const mpffn_field *ff)
{
  mp_ptr q=ff->tmp->mg_q;
  mp_ptr xx=ff->tmp->mg_xx;
  long i;
  xmpn_zero(xx,ff->lp);
  xx+=(l-1)*ff->lp;
  for(i=1;i<l;i++,xx-=ff->lp,x+=ff->lp)
  {
    if (!xmpn_is_zero(x,ff->lp))
      mpn_sub_n(xx,ff->p,x,ff->lp);
    else 
      xmpn_zero(xx,ff->lp);
  }
  mpffn_one(q,ff);
  for(i=0;i<l-1;i++)
  {
    mpffn_pol_mul(q,xx,l-1,q,l-1,ff);
    mpffn_add_ui(q,q,1,ff);
  }
  mpffn_pol_copy(r+ff->lp,q,l-1,ff);
  xmpn_zero(r,ff->lp);
}
