#include "mpffn.h"
#include "mpffn_priv.h"
#include "xmpn.h"

/*
@function: mpffn_mul
@section: mpffn/arith
@proto: void mpffn_mul(mp_ptr r, mp_srcptr x, mp_srcptr y, const mpffn_field *ff)
@temp: mul=ff->size_sqr
@desc: Set $r$ to $xy$ in \ff.
*/

void mpffn_mul(mp_ptr r, mp_srcptr x, mp_srcptr y, const mpffn_field *ff)
{
  mp_ptr z=ff->tmp->mul;
  long lx=mpffn_length(x,ff);
  long ly=mpffn_length(y,ff);
  lx=ff->lt;ly=ff->lt;
  if (!lx || !ly) 
  {
    mpffn_zero(r,ff);
    return;
  }
  mpffn_pol_mul(z, x, lx, y, ly, ff);
  mpffn_red(r,z,lx+ly-1,ff);
}

/*
@function: mpffn_pol_mul
@section: mpffn/pol
@proto: void mpffn_pol_mul(mp_ptr r, mp_srcptr x, mp_size_t lx, mp_srcptr y, mp_size_t ly, const mpffn_field *ff)
@temp: polmul=ff->size_sqr
@desc: Set $\{r,lx+ly-1\}$ to $\{x,lx\}\times\{y,ly\}$ in \fX.
*/

void mpffn_pol_mul(mp_ptr r, mp_srcptr x, mp_size_t lx, mp_srcptr y, mp_size_t ly, const mpffn_field *ff)
{
  mp_ptr z=ff->tmp->polmul;
  if (lx>=ly)
    mpn_mul(z,x,lx*ff->lp,y,ly*ff->lp);
  else
    mpn_mul(z,y,ly*ff->lp,x,lx*ff->lp);
  mpffn_pol_canon(r,z,lx+ly-1,ff);
}
