#include "mpffn.h"
#include "mpffn_priv.h"
#include "xmpn.h"

/*
@function: mpffn_mul_ui
@section: mpffn/arith
@proto: void mpffn_mul_ui(mp_ptr r, mp_srcptr x, unsigned long int y, const mpffn_field *ff)
@desc: Set $r$ to $xy$ in \ff.
*/

void mpffn_mul_ui(mp_ptr r, mp_srcptr x, unsigned long int y, const mpffn_field *ff)
{
    mpn_mul_1(r,x,ff->size_elt,y);
    mpffn_canon(r,r,ff);
}

/*
@function: mpffn_pol_mul_ui
@section: mpffn/pol
@proto: void mpffn_pol_mul_ui(mp_ptr r, mp_srcptr x, mp_size_t lt, unsigned long int y, const mpffn_field *ff)
@desc: Set $\{r,lt\}$ to $\{x,lt\}\times y$ in \fX.
*/

void mpffn_pol_mul_ui(mp_ptr r, mp_srcptr x, mp_size_t lt, unsigned long int y, const mpffn_field *ff)
{
    if (!lt) return;
    mpn_mul_1(r,x,lt*ff->lp,y);
    mpffn_pol_canon(r,r,lt,ff);
}


/*
@function: mpffn_mul_mpn
@section: mpffn/arith
@proto: void mpffn_mul_mpn(mp_ptr r, mp_srcptr x, mp_srcptr y, const mpffn_field *ff)
@temp: mul_n=ff->size_sqr+ff->size_p
@desc: Set $r$ to $x\times\{y,$\verb!ff->size_p!$\}$ in \ff,
where $\{y,$\verb!ff->size_p!$\}$ is understood as a \textit{mpn} operand.
*/

void mpffn_mul_mpn(mp_ptr r, mp_srcptr x, mp_srcptr y, const mpffn_field *ff)
{
    mp_ptr z=ff->tmp->mul_n;
    mpn_mul(z,x,ff->size_elt,y,ff->size_p);
    mpffn_pol_canon(r,z,ff->lt,ff);
}  

/*
@function: mpffn_pol_mul_mpn
@section: mpffn/pol
@proto: void mpffn_pol_mul_mpn(mp_ptr r, mp_srcptr x, mp_size_t lt, mp_srcptr y, const mpffn_field *ff)
@temp: mul_n=ff->size_sqr+ff->size_p
@desc: Set $\{r,lt\}$ to $\{x,lt\}\times\{y,$\verb!ff->size_p!$\}$ in \fX,
where $\{y,$\verb!ff->size_p!$\}$ is understood as a \textit{mpn} operand.
*/

void mpffn_pol_mul_mpn(mp_ptr r, mp_srcptr x, mp_size_t lt, mp_srcptr y, const mpffn_field *ff)
{
    mp_ptr z=ff->tmp->mul_n;
    if (!lt) return;
    mpn_mul(z,x,lt*ff->lp,y,ff->size_p);
    mpffn_pol_canon(r,z,lt,ff);
}  

/*
@function: mpffn_mul_z
@section: mpffn/arith
@proto: void mpffn_mul_z(mp_ptr r, mp_srcptr x, mpz_t y, const mpffn_field *ff)
@temp: canon1=ff->lp
       mul_z=ff->size_p
@desc: Set $r$ to $xy$ in \ff.
*/

void mpffn_mul_z(mp_ptr r, mp_srcptr x, mpz_t y, const mpffn_field *ff)
{
  mp_ptr z;
  if (y->_mp_size!=ff->size_p)
    z=y->_mp_d;
  else 
  {
    mp_ptr q=ff->tmp->canon1;
    z=ff->tmp->mul_z;
    mpn_tdiv_qr(q,z,0,y->_mp_d,y->_mp_size,ff->p,ff->size_p);
  }
  mpffn_mul_mpn(r, x, z, ff);
}  
