#include "mpffn.h"
#include "xmpn.h"

/*
@function: mpffn_unpack
@section: mpffn/assign
@proto: void mpffn_unpack(mp_ptr r, mp_srcptr x, mp_size_t packet_size, mp_size_t packet_number, const mpffn_field *ff)
@desc: Set $r$ to $x$ in \ff.
*/

void
mpffn_unpack(mp_ptr r, mp_srcptr x, mp_size_t packet_size, mp_size_t packet_number, const mpffn_field *ff)
{
  mp_size_t ppb=packet_size/mp_bits_per_limb;
  mp_size_t pmb=packet_size-mp_bits_per_limb*ppb;
  mp_size_t ppbu=min(ppb+(pmb>0),ff->lp);
  long i,j;
  for(i=0,j=0;j<packet_number;j++,i+=packet_size,r+=ff->lp)
  {
    long dec=i/mp_bits_per_limb;
    long res=i-dec*mp_bits_per_limb;
    if (res)
      mpn_rshift(r,x+dec,ppbu,res);
    else 
      xmpn_copy(r,x+dec,ppbu);
    if (pmb) r[ppb]&=(1UL<<pmb)-1;
    xmpn_zero(r+ppb+1,ff->lp-ppb-1);
  }
}
