#include "mpffn.h"
#include "mpffn_priv.h"
#include "xmpn.h"

/* This code is based on :
*/

/* mpn_pow_1 -- Compute powers R = U^exp.

   THE FUNCTIONS IN THIS FILE ARE FOR INTERNAL USE ONLY.  THEY'RE ALMOST
   CERTAIN TO BE SUBJECT TO INCOMPATIBLE CHANGES OR DISAPPEAR COMPLETELY IN
   FUTURE GNU MP RELEASES.

Copyright 2002 Free Software Foundation, Inc.

This file is part of the GNU MP Library.

The GNU MP Library is free software; you can redistribute it and/or modify it
under the terms of the GNU Lesser General Public License as published by the
Free Software Foundation; either version 2.1 of the License, or (at your
option) any later version.

The GNU MP Library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
for more details.

You should have received a copy of the GNU Lesser General Public License along
with the GNU MP Library; see the file COPYING.LIB.  If not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.  */




/*
@function: mpffn_pow_ui
@section: mpffn/arith
@proto: void mpffn_pow_ui(mp_ptr r, mp_srcptr x, mp_limb_t y, const mpffn_field *ff)
@temp: pow=ff->size_elt
@desc: Set $r$ to $x^y$.
*/

void
mpffn_pow_ui(mp_ptr r, mp_srcptr x, mp_limb_t y, const mpffn_field *ff)
{
  mp_limb_t yy;
  mp_ptr t=ff->tmp->pow,z;
  int cnt, i;
  int par;

  if (y <= 1)
  {
    if (y == 0)
      mpffn_one(r, ff);
    else
      mpffn_copy(r, x, ff);
    return;
  }

  par = 0;
  cnt = GMP_LIMB_BITS;
  for (yy = y; yy != 0; yy >>= 1)
  {
    par ^= yy & 1;
    cnt--;
  }
  y <<= cnt;

  if (((par ^ cnt) & 1) == 0)
  {
    z=r;r=t;t=z;
  }
  mpffn_mul (r, x, x, ff);

  for (i = GMP_LIMB_BITS - cnt - 1;;)
  {
    y <<= 1;
    if ((y & (1UL<<(GMP_LIMB_BITS-1))) != 0)
    {
      mpffn_mul (t, r, x, ff);
      z=r;r=t;t=z;
    }

    if (--i == 0)
      break;

    mpffn_mul (t, r, r, ff);
    z=r;r=t;t=z;
  }
}
