#include "mpffn.h"
#include "mpffn_priv.h"
#include <stdlib.h>
#include "xmpn.h"

size_t
xmpn_out_str(FILE *fout,int base, mp_srcptr x, mp_size_t n)
{
  char *s=malloc(mp_bits_per_limb*n*28/93+10);
  mp_ptr xc=calloc(mp_bits_per_limb/8, n+1);
  mp_size_t t;
  long i,j;
  xmpn_copy(xc,x,n);
  t = mpn_get_str(s,base,xc,n);
  for (j=0;s[j]==0 && j<t;j++);
  for (i=j;i<t;i++) 
    s[i]+= (s[i]<=9?'0':'A');
  s[t]=0;
  fputs(s+j, fout);
  free(s);
  free(xc);
  return t-j;
}

/*
@function: mpffn_out_str
@section: mpffn/io
@proto: size_t mpffn_out_str(FILE *STREAM, int base, mp_srcptr x, const mpffn_field *ff)
@desc: Output x on stdio stream STREAM, in GP format.  This function is currently only for debugging purpose.  Return the number of bytes written, or if an error occurred, return $0$.
*/

size_t
mpffn_out_str(FILE *fout, int base, mp_srcptr x, const mpffn_field *ff)
{
  long i;
  size_t s=0;
  long first=1;
  if (!mpffn_sgn(x,ff))
  {
    fprintf(fout,"0");
    return 1;
  }
  x += ff->impl->leader;
  for(i=ff->lt-1; i>=0; i--, x -= ff->lp)
  {
    if (xmpn_is_zero(x,ff->lp))
      continue;
    if (!first)
        s += fprintf(fout," + ");
    if (!xmpn_is_one(x,ff->lp))
    {
      s += xmpn_out_str(fout,base,x,ff->lp);
      if (i)
        s += fprintf(fout,"*");
    }
    else
      if (!i)
        s += fprintf(fout,"1");
    first=0;
    switch(i)
    {
    case 0: break;
    case 1: s += fprintf(fout,"x"); break;
    default: s += fprintf(fout,"x^%ld",i); break;
    }
  }
  return s;
}
/*
@function: mpffn_pol_out_str
@section: mpffn/pol
@proto: size_t mpffn_pol_out_str(FILE *STREAM, int base, mp_srcptr x, mp_size_t l, const mpffn_field *ff)
@desc: Output $\{x,l\}$ on stdio stream STREAM, in GP format.  This function is currently only for debugging purpose.  Return the number of bytes written, or if an error occurred, return $0$.
*/

size_t
mpffn_pol_out_str(FILE *fout, int base, mp_srcptr x, mp_size_t l, const mpffn_field *ff)
{
  long i;
  size_t s=0;
  long first=1;
  if (!mpffn_pol_sgn(x,l,ff))
  {
    fprintf(fout,"0");
    return 1;
  }
  x += (l-1)*ff->lp;
  for(i=l-1; i>=0; i--, x -= ff->lp)
  {
    if (xmpn_is_zero(x,ff->lp))
      continue;
    if (!first)
        s += fprintf(fout," + ");
    if (!xmpn_is_one(x,ff->lp))
    {
      s += xmpn_out_str(fout,base,x,ff->lp);
      if (i)
        s += fprintf(fout,"*");
    }
    else
      if (!i)
        s += fprintf(fout,"1");
    first=0;
    switch(i)
    {
    case 0: break;
    case 1: s += fprintf(fout,"x"); break;
    default: s += fprintf(fout,"x^%ld",i); break;
    }
  }
  return s;
}
